import java.util.concurrent.Callable;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

class MxNet2License implements Callable<Integer> {

    private static int USERCODE = 1234;
    private static short APPSLOT = 7;
    private static int INTERVAL=5000;   //ms

    private static short DNG_Port=85;
    private static short DNG_Nr=1;

    private boolean initOk=false;
    private boolean bExit=false;
    private Thread  loginUpdate=null;

    private static final boolean macOS = System.getProperty("os.name").contains("Mac OS");

	    public Integer call() throws Exception {
            int bOk=0;

            int[] DataIn = new int[32];
            
            short ret = MxNet2.rInit_MatrixAPI();
            System.out.println( "ret=" + ret);
            if( ret < 0) return bOk;
            
            ret = MxNet2.rDongle_Count(DNG_Port);
            System.out.println( "Count=" + ret);
            //ret==0（ドングルカウント 0 )はエラー
            if( ret <= 0) {
                MxNet2.rRelease_MatrixAPI();
                return bOk;
            }

            int SerNr = MxNet2.rDongle_ReadSerNr(USERCODE, DNG_Nr, DNG_Port);
            System.out.println( "SerNr=" + SerNr);
            if( SerNr <= 0) {
                MxNet2.rRelease_MatrixAPI();
                return bOk;
            }

            ret = MxNet2.rDongle_ReadData(USERCODE, DataIn, (short)16, DNG_Nr, DNG_Port);
            System.out.println( "Read Count=" + ret );
            if( ret != 16){
                MxNet2.rRelease_MatrixAPI();
                return bOk;
            }

            for( int i=0 ; i < 16 ; i++)
                System.out.println("   Field" + i + ":" + DataIn[i]);

            // LogIn_MatrixNet
            ret = MxNet2.rLogIn_MatrixNet(USERCODE, APPSLOT, DNG_Nr);
            System.out.println( "Login=" + ret );
            //ret==0（ライセンス残 0 )は成功
            if( ret < 0) {
                MxNet2.rRelease_MatrixAPI();
                return bOk;
            }

            bOk=1;
            return bOk;
	}

    public boolean get()
    {
        //プログラムでIP/Portを指定すれば mxnetapi.ini が見つからなくても
        //サーバに接続可能
        String setip = MxNet2.MxNet_SetIP("mxnet2.local");
        if( setip == "")
        {
            System.out.println("failed to set ip");
        }
        else
        {
           System.out.println("Server IP = " + setip);
           MxNet2.MxNet_SetPort(12300);
        }


        //メインスレッドとは別スレッド（ スレッドや Executor Service等）で
        //MxNet2サーバとやり取りする
        if( macOS == true )
        {
            System.out.println("Running on non Mac OS");

            Thread th = new Thread( () -> {

                int[] DataIn = new int[32];
                
                short ret = MxNet2.rInit_MatrixAPI();
                System.out.println( "ret=" + ret);
                if( ret < 0) return;
                
                ret = MxNet2.rDongle_Count(DNG_Port);
                System.out.println( "Count=" + ret);
                //ret==0（ドングルカウント 0 )はエラー
                if( ret <= 0) {
                    MxNet2.rRelease_MatrixAPI();
                    return;
                }

                int SerNr = MxNet2.rDongle_ReadSerNr(USERCODE, DNG_Nr, DNG_Port);
                System.out.println( "SerNr=" + SerNr);
                if( SerNr <= 0) {
                    MxNet2.rRelease_MatrixAPI();
                    return;
                }

                ret = MxNet2.rDongle_ReadData(USERCODE, DataIn, (short)16, DNG_Nr, DNG_Port);
                System.out.println( "Read Count=" + ret );
                if( ret != 16){
                    MxNet2.rRelease_MatrixAPI();
                    return;
                }

                for( int i=0 ; i < 16 ; i++)
                    System.out.println("   Field" + i + ":" + DataIn[i]);

                // LogIn_MatrixNet
                ret = MxNet2.rLogIn_MatrixNet(USERCODE, APPSLOT, DNG_Nr);
                System.out.println( "Login=" + ret );
                //ret==0（ライセンス残 0 )は成功
                if( ret < 0) {
                    MxNet2.rRelease_MatrixAPI();
                    return;
                }

                initOk=true;
            } );

            th.start();
            
            try{
                th.join();
            }catch(InterruptedException e)
            {
                e.printStackTrace();
            }
        }
        //macOS では ExecutorService が動かない
        else
        {
            System.out.println("Running on non Mac OS");

            int ok=0;

            ExecutorService execService = null;
            try
             {
                execService = Executors.newSingleThreadExecutor();
                Future<Integer> result = execService.submit(this);
                try {
                    ok = result.get();
                } catch (InterruptedException | ExecutionException e) {
                    e.printStackTrace();
                }
            }
            finally
            {
                execService.shutdown();
            }

            if( ok==0) return false;
        }

        //Start periodic Login Update
        loginUpdate = new Thread( () -> {
           while(true)
            {
                synchronized(this)
                {
                    try{
                        wait(INTERVAL);
                    }
                    catch(InterruptedException e)
                    {

                    }
                }
                    
               if( bExit==true) break;

                short ret = MxNet2.rLogIn_MatrixNet(USERCODE, APPSLOT, DNG_Nr); 
                System.out.println( "Login=" + ret );

                if( ret < 0) 
                {
                    MxNet2.rRelease_MatrixAPI();
                    System.exit(-1);
                }
            }

       } );
       loginUpdate.start();

        return true;
    }

    public void release()
    {
        bExit=true;

        synchronized(this)
        {
            notify();
        }

        try{
            loginUpdate.join();
        }catch(InterruptedException e)
        {
            e.printStackTrace();
        }

        MxNet2.rRelease_MatrixAPI();

    }

}

